//
//  HSChilkatSSHTunnel.h
//  HSCore
//
//  Created by Kyle Hankinson on 2019-04-18.
//  Copyright © 2019 Hankinsoft Development, Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

// Do not change these values. They are used to save information to
// our users keychain.
#define SSHTunnelErrorDomain                    @"com.hankinsoft.sqlpro.sshtunnel"
#define SSHTunnelHostDataKey                    (SSHTunnelErrorDomain @".hostdatakey")
#define SSHTunnelFingerprintKey                 (SSHTunnelErrorDomain @".fingerprintkey")
#define SSHTunnelUnknownFingerprintErrorCode    (1024)

typedef NSString * _Nullable (^HSSSHTunnelRequestPasswordBlock) (void);
typedef NSArray* _Nullable (^HSSSHTunnelRequestPromptsBlock) (NSArray * prompts);

@class HSChilkatSSHTunnelHelper;

@protocol HSChilkatSSHFingerprintDelegate

- (NSData*) SSHTunnel: (HSChilkatSSHTunnelHelper*) tunnelHelper
           keyForHost: (NSString*) host
                 port: (NSUInteger) port;

- (void) SSHTunnel: (HSChilkatSSHTunnelHelper*) tunnelHelper
setPromptResponses: (NSArray<NSString*>*) responses
     forIdentifier: (NSString*) identifier;

- (NSArray<NSString*>*) SSHTunnel: (HSChilkatSSHTunnelHelper*) tunnelHelper
  getPromptResponsesForIdentifier: (NSString*) identifier;

@end

@protocol HSChilkatSSHTunnelDelegate <NSObject>

- (NSString*) SSHTunnel: (HSChilkatSSHTunnelHelper*) tunnelHelper
        requestPasswordForUser: (NSString*) username;

- (NSURL*)   SSHTunnel: (HSChilkatSSHTunnelHelper*) tunnelHelper
      requestSecureURL: (NSString*) filePath;

- (NSArray<NSString*>*) SSHTunnel: (HSChilkatSSHTunnelHelper*) tunnelHelper
   requestKeyboardAuthWithPrompts: (NSArray<NSString*>*) prompts
                        cancelled: (BOOL*) pCancelled;

- (NSURL*)   SSHTunnel: (HSChilkatSSHTunnelHelper*) tunnelHelper
requestPublicKeyAccess: (NSURL*) url;

@end

@interface HSChilkatSSHTunnelHelper : NSObject

+ (void) setFingerprintDelegate: (id<HSChilkatSSHFingerprintDelegate>) fingerprintDelegate;
+ (BOOL) validateSSHPrivateKey: (NSData*) sshPrivateKeyData
                      password: (NSString*) sshPrivateKeyPassword
                         error: (NSError**) pError;

+ (NSArray<NSString*>*) promptsFromInteractiveKeyboardAuthXML: (NSString*) xml
                                                        error: (NSError**) pError;

+ (NSString*) responseStringForInteractiveKeyboardAuthResponses: (NSArray<NSString*>*) prompts
                                                          error: (NSError**) pError;

- (BOOL) connect;
- (void) disconnect;

@property(nonatomic,weak)   id representedObject;

@property(nonatomic,copy)   NSString * sshHostName;
@property(nonatomic,copy)   NSString * sshPassword;
@property(nonatomic,assign) BOOL     sshRememberPassword;
@property(nonatomic,copy)   NSString * sshUsername;

@property(nonatomic,assign) NSUInteger sshPort;

@property(nonatomic,copy)   NSString * forwardToHost;
@property(nonatomic,assign) NSUInteger forwardToPort;

@property(nonatomic,copy)   NSString * sshPrivateKeyPath;
@property(nonatomic,retain) NSData   * sshPrivateKeyData;
@property(nonatomic,copy)   NSString * sshPrivateKeyPassword;

@property(nonatomic,weak) id<HSChilkatSSHTunnelDelegate> tunnelDelegate;

// Note: Should only be used for debugging purpose
@property(nonatomic,assign) BOOL sshIgnoreHostKey;

- (NSUInteger) localPort;
- (NSError*) error;

@end

NS_ASSUME_NONNULL_END
